@@ -1,5 +1,5 @@
 /* Tcl/Tk command definitions for Insight.
-   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2001, 2002, 2003, 2004
+   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2001, 2002, 2003, 2004, 2007, 2008
    Free Software Foundation, Inc.
 
    Written by Stu Grossman <grossman@cygnus.com> of Cygnus Support.
@@ -39,13 +39,18 @@
 #include "filenames.h"
 #include "disasm.h"
 #include "value.h"
+#include "varobj.h"
 #include "exceptions.h"
+#include "language.h"
+#include "target.h"
+#include "valprint.h"
+#include "regcache.h"
+#include "arch-utils.h"
 
 /* tcl header files includes varargs.h unless HAS_STDARG is defined,
    but gdb uses stdarg.h, so make sure HAS_STDARG is defined.  */
 #define HAS_STDARG 1
 
-#include <itcl.h>
 #include <tcl.h>
 #include <tk.h>
 
@@ -56,9 +61,11 @@
 
 #include <signal.h>
 #include <fcntl.h>
+#ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
+#endif
 #include <sys/time.h>
-#include <sys/stat.h>
+#include "gdb_stat.h"
 
 #include "gdb_string.h"
 #include "dis-asm.h"
@@ -452,7 +459,8 @@ gdb_clear_file (ClientData clientData, Tcl_Interp *interp,
 
   if (! ptid_equal (inferior_ptid, null_ptid) && target_has_execution)
     {
-      if (attach_flag)
+      struct inferior *inf = current_inferior ();
+      if (inf->attach_flag)
 	target_detach (NULL, 0);
       else
 	target_kill ();
@@ -563,8 +571,8 @@ gdb_stop (ClientData clientData, Tcl_Interp *interp,
     }
   else
     {
-      if (target_stop != target_ignore)
-	target_stop ();
+      if (target_ignore != (void (*) (void)) current_target.to_stop)
+	target_stop (gdbtk_get_ptid ());
       else
 	quit_flag = 1;		/* hope something sees this */
     }
@@ -609,6 +617,7 @@ gdb_eval (ClientData clientData, Tcl_Interp *interp,
   struct ui_file *stb;
   long dummy;
   char *result;
+  struct value_print_options opts;
 
   if (objc != 2 && objc != 3)
     {
@@ -619,6 +628,8 @@ gdb_eval (ClientData clientData, Tcl_Interp *interp,
   if (objc == 3)
     format = *(Tcl_GetStringFromObj (objv[2], NULL));
 
+  get_formatted_print_options (&opts, format);
+
   expr = parse_expression (Tcl_GetStringFromObj (objv[1], NULL));
   old_chain = make_cleanup (free_current_contents, &expr);
   val = evaluate_expression (expr);
@@ -627,8 +638,8 @@ gdb_eval (ClientData clientData, Tcl_Interp *interp,
   stb = mem_fileopen ();
   make_cleanup_ui_file_delete (stb);
   val_print (value_type (val), value_contents (val),
-	     value_embedded_offset (val), VALUE_ADDRESS (val),
-	     stb, format, 0, 0, 0);
+	     value_embedded_offset (val), value_address (val),
+	     stb, 0, &opts, current_language);
   result = ui_file_xstrdup (stb, &dummy);
   Tcl_SetObjResult (interp, Tcl_NewStringObj (result, -1));
   xfree (result);
@@ -697,7 +708,7 @@ gdb_cmd (ClientData clientData, Tcl_Interp *interp,
       result_ptr->flags |= GDBTK_TO_RESULT;
     }
 
-  bpstat_do_actions (&stop_bpstat);
+  bpstat_do_actions ();
 
   return TCL_OK;
 }
@@ -748,7 +759,7 @@ gdb_immediate_command (ClientData clientData, Tcl_Interp *interp,
 
   execute_command (Tcl_GetStringFromObj (objv[1], NULL), from_tty);
 
-  bpstat_do_actions (&stop_bpstat);
+  bpstat_do_actions ();
 
   result_ptr->flags |= GDBTK_TO_RESULT;
 
@@ -1491,7 +1502,7 @@ gdb_listfuncs (clientData, interp, objc, objv)
 		}
 	      else
 		{
-		  funcVals[0] = Tcl_NewStringObj (DEPRECATED_SYMBOL_NAME (sym), -1);
+		  funcVals[0] = Tcl_NewStringObj (SYMBOL_PRINT_NAME (sym), -1);
 		  funcVals[1] = not_mangled;
 		}
 	      Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr,
@@ -1802,7 +1813,7 @@ gdbtk_load_source (ClientData clientData, struct symtab *symtab,
 	      /* FIXME: Convert to Tcl_SetVar2Ex when we move to 8.2.  This
 		 will allow us avoid converting widget_line_no into a string. */
 	      
-	      xasprintf (&buffer, "%d", client_data->widget_line_no);
+	      buffer = xstrprintf ("%d", client_data->widget_line_no);
 	      
 	      Tcl_SetVar2 (client_data->interp, client_data->map_arr,
 			   Tcl_DStringValue (&client_data->src_to_line_prefix),
@@ -1885,7 +1896,7 @@ gdbtk_load_asm (ClientData clientData, CORE_ADDR pc,
   for (i = 0; i < 3; i++)
     Tcl_SetObjLength (client_data->result_obj[i], 0);
 
-  deprecated_print_address_numeric (pc, 1, gdb_stdout);
+  fputs_filtered (paddress (get_current_arch (), pc), gdb_stdout);
   gdb_flush (gdb_stdout);
 
   result_ptr->obj_ptr = client_data->result_obj[1];
@@ -1894,7 +1905,7 @@ gdbtk_load_asm (ClientData clientData, CORE_ADDR pc,
 
   result_ptr->obj_ptr = client_data->result_obj[2];
   /* FIXME: cagney/2003-09-08: This should use gdb_disassembly.  */
-  insn = gdb_print_insn (pc, gdb_stdout);
+  insn = gdb_print_insn (get_current_arch (), pc, gdb_stdout, NULL);
   gdb_flush (gdb_stdout);
 
   client_data->widget_line_no++;
@@ -1918,7 +1929,7 @@ gdbtk_load_asm (ClientData clientData, CORE_ADDR pc,
       /* FIXME: Convert to Tcl_SetVar2Ex when we move to 8.2.  This
 	 will allow us avoid converting widget_line_no into a string. */
       
-      xasprintf (&buffer, "%d", client_data->widget_line_no);
+      buffer = xstrprintf ("%d", client_data->widget_line_no);
       
       Tcl_SetVar2 (client_data->interp, client_data->map_arr,
 		   Tcl_DStringValue (&client_data->pc_to_line_prefix),
@@ -2129,23 +2140,32 @@ gdb_loc (ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST obj
           pc = entry_point_address ();
           sal = find_pc_line (pc, 0);
         }  
-      else if (deprecated_selected_frame
-	       && (get_frame_pc (deprecated_selected_frame) != read_pc ()))
-        {
-          /* Note - this next line is not correct on all architectures.
-	     For a graphical debugger we really want to highlight the 
-	     assembly line that called the next function on the stack.
-	     Many architectures have the next instruction saved as the
-	     pc on the stack, so what happens is the next instruction 
-	     is highlighted. FIXME */
-	  pc = get_frame_pc (deprecated_selected_frame);
-	  find_frame_sal (deprecated_selected_frame, &sal);
-	}
       else
-        {
-          pc = read_pc ();
-          sal = find_pc_line (pc, 0);
-        }
+	{
+	  struct frame_info *frame;
+	  CORE_ADDR frame_pc, current_pc;
+
+	  frame = get_selected_frame (NULL);
+	  current_pc = regcache_read_pc (get_current_regcache ());
+	  frame_pc = get_frame_pc (frame);
+ 
+	  if (frame_pc != current_pc)
+	    {
+	      /* Note - this next line is not correct on all architectures.
+		 For a graphical debugger we really want to highlight the 
+		 assembly line that called the next function on the stack.
+		 Many architectures have the next instruction saved as the
+		 pc on the stack, so what happens is the next instruction 
+		 is highlighted. FIXME */
+	      pc = frame_pc;
+	      find_frame_sal (frame, &sal);
+	    }
+	  else
+	    {
+	      pc = current_pc;
+	      sal = find_pc_line (pc, 0);
+	    }
+	}
     }
   else if (objc == 2)
     {
@@ -2252,7 +2272,7 @@ hex2bin (const char *hex, char *bin, int count)
   int incr = 2;
 
 
-  if (TARGET_BYTE_ORDER == BFD_ENDIAN_LITTLE)
+  if (gdbarch_byte_order (get_current_arch ()) == BFD_ENDIAN_LITTLE)
     {
       /* need to read string in reverse */
       hex += count - 2;
@@ -2429,7 +2449,9 @@ gdb_update_mem (ClientData clientData, Tcl_Interp *interp,
   memset (mbuf, 0, nbytes + 32);
   mptr = cptr = mbuf;
 
-  rnum = target_read (&current_target, TARGET_OBJECT_MEMORY, NULL,
+  /* Dispatch memory reads to the topmost target, not the flattened
+     current_target.  */
+  rnum = target_read (current_target.beneath, TARGET_OBJECT_MEMORY, NULL,
 		      mbuf, addr, nbytes);
   if (rnum <= 0)
     {
@@ -2445,23 +2467,23 @@ gdb_update_mem (ClientData clientData, Tcl_Interp *interp,
   switch (size)
     {
     case 1:
-      val_type = builtin_type_int8;
+      val_type = builtin_type (get_current_arch ())->builtin_int8;
       asize = 'b';
       break;
     case 2:
-      val_type = builtin_type_int16;
+      val_type = builtin_type (get_current_arch ())->builtin_int16;
       asize = 'h';
       break;
     case 4:
-      val_type = builtin_type_int32;
+      val_type = builtin_type (get_current_arch ())->builtin_int32;
       asize = 'w';
       break;
     case 8:
-      val_type = builtin_type_int64;
+      val_type = builtin_type (get_current_arch ())->builtin_int64;
       asize = 'g';
       break;
     default:
-      val_type = builtin_type_int8;
+      val_type = builtin_type (get_current_arch ())->builtin_int8;
       asize = 'b';
     }
 
@@ -2518,9 +2540,13 @@ gdb_update_mem (ClientData clientData, Tcl_Interp *interp,
 	}
       else
 	{
+	  struct value_print_options opts;
+
+	  get_formatted_print_options (&opts, format);
+
 	  /* print memory to our uiout file and set the table's variable */
 	  ui_file_rewind (stb);
-	  print_scalar_formatted (mptr, val_type, format, asize, stb);
+	  print_scalar_formatted (mptr, val_type, &opts, asize, stb);
 	  tmp = ui_file_xstrdup (stb, &dummy);
 
 	  /* See comments above on max_*_len */
@@ -2892,7 +2918,7 @@ gdbtk_set_result (Tcl_Interp *interp, const char *fmt,...)
   char *buf;
 
   va_start (args, fmt);
-  xvasprintf (&buf, fmt, args);
+  buf = xstrvprintf (fmt, args);
   va_end (args);
   Tcl_SetObjResult (interp, Tcl_NewStringObj (buf, -1));
   xfree(buf);
@@ -2986,7 +3012,10 @@ gdb_CA_to_TAS (ClientData clientData, Tcl_Interp *interp,
 
   /* This is not really correct.  Using paddr_nz() will convert to hex and truncate 
      to 32-bits when required but will otherwise not do what we really want. */
-  Tcl_SetStringObj (result_ptr->obj_ptr, paddr_nz (address), -1);
+  
+  Tcl_SetStringObj (result_ptr->obj_ptr,
+		    paddress (get_current_arch (), address),
+		    -1);
 
   return TCL_OK;
 }
@@ -2995,13 +3024,9 @@ gdb_CA_to_TAS (ClientData clientData, Tcl_Interp *interp,
  * with something similar, but different enough to break
  * Insight.
  */
-int find_and_open_source (struct objfile *objfile, const char *filename,
-			  const char *dirname, char **fullname);
-
 char *
 symtab_to_filename (struct symtab *s)
 {
-
   int r;
 
   if (!s)
@@ -3009,8 +3034,7 @@ symtab_to_filename (struct symtab *s)
 
   /* Don't check s->fullname here, the file could have been 
      deleted/moved/..., look for it again */
-  r = find_and_open_source (s->objfile, s->filename, s->dirname,
-			    &s->fullname);
+  r = open_source_file (s);
   if (r)
     close (r);
 
